package com.ruoyi.system.mapper;

import com.ruoyi.system.domain.EpsNodeBandwidth;

import java.util.List;

/**
 * 节点带宽信息Mapper接口
 * 
 * @author gyt
 * @date 2025-08-12
 */
public interface EpsNodeBandwidthMapper 
{
    /**
     * 查询节点带宽信息
     * 
     * @param id 节点带宽信息主键
     * @return 节点带宽信息
     */
    public EpsNodeBandwidth selectEpsNodeBandwidthById(Long id);

    /**
     * 查询节点带宽信息列表
     * 
     * @param epsNodeBandwidth 节点带宽信息
     * @return 节点带宽信息集合
     */
    public List<EpsNodeBandwidth> selectEpsNodeBandwidthList(EpsNodeBandwidth epsNodeBandwidth);

    /**
     * 新增节点带宽信息
     * 
     * @param epsNodeBandwidth 节点带宽信息
     * @return 结果
     */
    public int insertEpsNodeBandwidth(EpsNodeBandwidth epsNodeBandwidth);

    /**
     * 修改节点带宽信息
     * 
     * @param epsNodeBandwidth 节点带宽信息
     * @return 结果
     */
    public int updateEpsNodeBandwidth(EpsNodeBandwidth epsNodeBandwidth);

    /**
     * 删除节点带宽信息
     * 
     * @param id 节点带宽信息主键
     * @return 结果
     */
    public int deleteEpsNodeBandwidthById(Long id);

    /**
     * 批量删除节点带宽信息
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteEpsNodeBandwidthByIds(Long[] ids);

    /**
     * 计算月均日95值
     *
     * @param epsNodeBandwidth 节点带宽信息
     * @return 节点带宽信息
     */
    public EpsNodeBandwidth calculateAvg(EpsNodeBandwidth epsNodeBandwidth);
    /**
     * 月均日95值-相关数据
     *
     * @param epsNodeBandwidth 节点带宽信息
     * @return 节点带宽信息
     */
    public List<EpsNodeBandwidth> getAvgDetailMsg(EpsNodeBandwidth epsNodeBandwidth);
    /**
     * 查询月均日95值是否存在
     *
     * @param epsNodeBandwidth 节点带宽信息
     * @return 节点带宽信息
     */
    public int countByAvgMsg(EpsNodeBandwidth epsNodeBandwidth);

    int updateEpsNodeBandwidthByServerSn(EpsNodeBandwidth epsNodeBandwidth);

    int updateEpsNodeBandwidthBySwitchSn(EpsNodeBandwidth epsNodeBandwidth);
}
